/* vms specific utility routines... */

#define XTERN extern
#include <common.h>
#include <ssdef.h>

void
io_redirection(int *argc, char **argv[])
{
    int i, k, new_argc = 0;
    int append;
    char ** vms_argv;
    FILE *f;

    vms_argv = (char **) malloc((*argc+1) * sizeof(char*));

    vms_argv[new_argc++] = **argv;

    for (i = 1; i < *argc; i++) {
        if (argv[0][i][0] == '>') {
            k = 1;
            append = 0;
            if (argv[0][i][k] == '>') {
                k++;
                append++;
            }
            if (argv[0][i][k]) {
                f = freopen(argv[0][i]+k,append?"a":"w",stdout);
                if (!f) {
                    perror("stdout redirection");
                    exit(SS$_ABORT);
                }
            } else if (i+1 < *argc && argv[0][i+1] && *argv[0][i+1]) {
                f = freopen(argv[0][i+1],append?"a":"w",stdout);
                if (!f) {
                    perror("stdout redirection");
                    exit(SS$_ABORT);
                }
                i++;
            }
        } else if (argv[0][i][0] == '2' && argv[0][i][1] == '>') {
            k = 2;
            append = 0;
            if (argv[0][i][k] == '>') {
                k++;
                append++;
            }
            if (argv[0][i][k]) {
                f = freopen(argv[0][i]+k,append?"a":"w",stderr);
                if (!f) {
                    perror("stderr redirection");
                    exit(SS$_ABORT);
                }
            } else if (i+1 < *argc && argv[0][i+1] && *argv[0][i+1]) {
                f = freopen(argv[0][i+1],append?"a":"w",stderr);
                if (!f) {
                    perror("stderr redirection");
                    exit(SS$_ABORT);
                }
                i++;
            }
        } else if (*argv[0][i] == '<') {
            if (argv[0][i][1]) {
                f = freopen(argv[0][i]+1,"r",stdin);
                if (!f) {
                    perror("stdin redirection");
                    exit(SS$_ABORT);
                }
            } else if (i+1 < *argc && argv[0][i+1] && *argv[0][i+1]) {
                f = freopen(argv[0][i+1],"r",stdin);
                if (!f) {
                    perror("stdin redirection");
                    exit(SS$_ABORT);
                }
                i++;
            }
        } else {
            vms_argv[new_argc++] = argv[0][i];
        }
    }

    *argc = new_argc;
    vms_argv[new_argc] = NULL;
    *argv = vms_argv;
}


#if __VMS_VER < 70000000

int
strncasecmp(const char *s1, const char *s2, size_t n)
{
    int i;
    char c1, c2;

    for (i=0; i < n; i++) {
        c1 = toupper(s1[i]);
        c2 = toupper(s2[i]);
        if (c1 != c2) return c1 < c2 ? -1 : 1;
        if (c1 == '\0') return  c2 ? 1: 0; /* s2 is longer */
    }
    return 0;
}
#endif

/* there's apparantly a BUG in several of the CRTL functions for 
  dealing with unix-style filenames that have a filetype that begins
  with "-".  The most likely place for this to occur is a rename
  (i.e., renaming "CHANGES." to "CHANGES.-ORIG"), so
  we try and fix it here... by converting to VMS filenames./

    But note that we can't use the CRTL to convert *directly* for us,
    since it's broken...*sigh* 
*/


#include <unixlib.h>

static char *tmpname;

static int
vms_name_fetcher(char *name, int type)
{
    if( type == DECC$K_DIRECTORY  ||
        type == DECC$K_FOREIGN)  return 0;

    if (name[0] == '[' && name[1] == ']') {
        tmpname = malloc(strlen(name)+10);
        strcpy(tmpname,"SYS$DISK:");
        strcat(tmpname, name);
    } else {
        tmpname = malloc(strlen(name)+1);
        strcpy(tmpname, name);
    }
    return 1;
}

static int
unix_name_fetcher(char *name)
{
    tmpname = malloc(strlen(name)+1);
    strcpy(tmpname, name);
    return 1;
}


char *
vms_vmsify(char *name)
{
    char *p, *q;

    if (!name) return 0;

    if (strchr(name,'[') || strchr(name,':') || strchr(name,'<')) {
        p = malloc(strlen(name)+1);
        strcpy(p,name);
        return p;
    }

    if (!strchr(name,'/')) {
        p = malloc(strlen(name)+12);
        strcpy(p,"SYS$DISK:[]");
        strcat(p,name);
        return p;
    }

    tmpname = 0;
#if BROKEN_DOT_MINUS
    q = strstr(name,".-");
    if (!q) q = name+strlen(name);
    p = malloc(strlen(name)+1);
    strcpy(p,name);
    p[q-name] = '\0';

    if (decc$to_vms(p,vms_name_fetcher,0,0) != 1) {
        if (tmpname) free(tmpname);
        free(p);
        return 0;
    }

    p = realloc(p, strlen(tmpname)+strlen(q)+1);
    strcpy(p,tmpname);
    strcat(p,q);
    free(tmpname);
    return p;
#else
    if (decc$to_vms(name,vms_name_fetcher,0,0) != 1) {
        if (tmpname) free(tmpname);
        return 0;
    }
    return tmpname;
#endif
}


char *
vms_unixify(char *name)
{
    char *p, *q;
    int is_sub;

    if (!name) return 0;

    if (strchr(name,'/')) {
        p = malloc(strlen(name)+1);
        strcpy(p,name);
        return p;
    }

    if (!strchr(name,'[') && !strchr(name,'<') && !strchr(name,':')) {
        p = malloc(strlen(name)+2);
        strcpy(p,"./");
        strcat(p,name);
        return p;
    }

    if (strncasecmp(name,"SYS$DISK:[]",11) == 0) {
        p = malloc(strlen(name)-11+2+1);
        strcpy(p,"./");
        strcat(p,name+11);
        return p;
    }
    is_sub = strncmp(name,"[.",2) == 0;

#if BROKEN_DOT_MINUS
    q = strrchr(name,']');
    if (!q) q = name;
    q = strrchr(q,'>');
    if (!q) q = name;

    q = strstr(q,".-");
    if (!q) q = name+strlen(name);
    p = malloc(strlen(name)+1);
    strcpy(p,name);
    p[q-name] = '\0';

    tmpname = 0;
    if (decc$from_vms(p,unix_name_fetcher,0) != 1) {
        if (tmpname) free(tmpname);
        free(p);
        return 0;
    }

    p = realloc(p, strlen(tmpname)+strlen(q)+1);
    if (is_sub && *tmpname == '/') {
        strcpy(p,tmpname+1);
    } else {
        strcpy(p,tmpname);
    }
    strcat(p,q);
    free(tmpname);
    return p;
#else
    tmpname = 0;
    if (decc$from_vms(name,unix_name_fetcher,0) != 1) {
        if (tmpname) free(tmpname);
        return 0;
    }
    if (is_sub && *tmpname == '/') {
        p = malloc(strlen(tmpname));
        strcpy(p,tmpname+1);
        free(tmpname);
        return p;
    }
    return tmpname;
#endif
}




